#!/usr/bin/env python2
import datetime
import os
import sys

import matplotlib.pyplot as plt
from matplotlib.ticker import FixedLocator, FormatStrFormatter
from mutagen.mp3 import MP3
import numpy

try:
    import cPickle as pickle
except:
    import pickle

def process_year(year, filename):
    if year:
        year = year.text[0].text
        # handle '2005-09-27'
        year = year.split('-')[0]
        # and ensure it wasn't a blank string
        if year:
            year = int(year)
            if year < 1900 or year > 2020:
                print "invalid year: %s, %s" % (year, filename)
                return None
            return year
    return None

def mp3_bitrates(filename, data):
    (root, ext) = os.path.splitext(filename)
    if ext == '.mp3':
        audio = MP3(filename)
        bitrate = audio.info.bitrate
        year = process_year(audio.get('TDRC', None), filename)
        data.append( (bitrate / 1000.0, year) )

def get_data(dirpath):
    data = []
    for root, dirs, files in os.walk(dirpath):
        for name in files:
            fullpath = os.path.join(root, name)
            mp3_bitrates(fullpath, data)
    return data

def plot_data(data):
    # unzip the (bitrate, year) tuples into two lists
    bitrate_stack, year_stack = zip(*data)

    valid_bitrates = [32, 40, 48, 56, 64, 80, 96, 112, 128,
            144, 160, 192, 224, 256, 320]
    bitrate_bins = [0] + [br + 1 for br in valid_bitrates]
    # fake unknown year songs to one below minimum
    default_year = min(year for year in year_stack if year is not None) - 1
    year_stack = [year if year is not None else default_year \
            for year in year_stack]

    # attempt to create one bin per year in range
    year_bins = range(min(year_stack) - 1, max(year_stack) + 1)
    year_bins = [year + 0.5 for year in year_bins]
    histo, xedges, yedges = numpy.histogram2d(bitrate_stack, year_stack,
            bins=[bitrate_bins, year_bins])

    # plot time
    x_coords, y_coords = numpy.meshgrid(year_bins, bitrate_bins)
    plt.pcolor(x_coords, y_coords, histo)
    axes = plt.gca()
    axes.xaxis.set_major_formatter(FormatStrFormatter('%d'))
    axes.set_xlim(year_bins[0], datetime.datetime.now().year + 1)
    axes.yaxis.set_major_locator(FixedLocator([0] + valid_bitrates))
    axes.set_ylim(0, 320)
    plt.xlabel('Song Year')
    plt.ylabel('Bitrate (kbps)')
    plt.hot()
    plt.colorbar()
    plt.show()

if __name__ == '__main__':
    arg = sys.argv[1]
    if os.path.isdir(arg):
        data = get_data(arg)
        # save data if a filename was provided as second argument
        if len(sys.argv) > 2:
            output = open(sys.argv[2], 'w')
            pickle.dump(data, output, pickle.HIGHEST_PROTOCOL)
            output.close()
    elif os.path.isfile(arg):
        datafile = open(arg)
        data = pickle.load(datafile)
        datafile.close()
    else:
        print "No directory or data provided!"
        sys.exit(1)

    plot_data(data)
